searchData={"items":[{"type":"module","title":"Modulex","doc":"","ref":"Modulex.html"},{"type":"macro","title":"Modulex.defbehaviour/1","doc":"","ref":"Modulex.html#defbehaviour/1"},{"type":"macro","title":"Modulex.defimplementation/1","doc":"","ref":"Modulex.html#defimplementation/1"},{"type":"type","title":"Modulex.t/0","doc":"","ref":"Modulex.html#t:t/0"},{"type":"module","title":"Modulex.Macros","doc":"This module contains all the macros privately used by the `Modulex` module.\nBecause we don't want these macros to be user-facing when users do `use Modulex`,\nwe extract them into a different module.","ref":"Modulex.Macros.html"},{"type":"macro","title":"Modulex.Macros.define_behaviour_functions/1","doc":"This macro returns an AST that defines the functions that are defined in the behaviour module.","ref":"Modulex.Macros.html#define_behaviour_functions/1"},{"type":"function","title":"Modulex.Macros.ensure_module_loaded!/1","doc":"This macro ensures the given module has been loaded.","ref":"Modulex.Macros.html#ensure_module_loaded!/1"},{"type":"macro","title":"Modulex.Macros.functions/0","doc":"This macro returns an AST containing a set of utility functions\nthat need to be incorporated into the module that uses the `Modulex` module.","ref":"Modulex.Macros.html#functions/0"},{"type":"macro","title":"Functions - Modulex.Macros.functions/0","doc":"- `mock_module/0`: Returns the mock module.\n- `implementation_module/0`: Returns the implementation module.\n- `behaviour_module/0`: Returns the behaviour module.\n- `get_application_env_module/0`: Returns the module that has been set in the environment (if any).","ref":"Modulex.Macros.html#functions/0-functions"},{"type":"module","title":"Modulex.MapUtils","doc":"A set of utilities to work with maps","ref":"Modulex.MapUtils.html"},{"type":"function","title":"Modulex.MapUtils.put_deep/3","doc":"","ref":"Modulex.MapUtils.html#put_deep/3"},{"type":"extras","title":"modulex","doc":"# `modulex`\n\nThe Application Module serves as a utility library designed to facilitate the creation of modules that conform to a specific behavior. Additionally, it allows for the behavior implementation to be managed through the application's configuration settings. This library is particularly useful when paired with testing libraries like [Mox](https://github.com/dashbitco/mox) or [Hammox](https://github.com/msz/hammox), as it helps eliminate the boilerplate code often required to retrieve the module ID from the application environment.\n\nThe following is an exampmle of an application module:\n\n```elixir\ndefmodule MyApplication.Module do\n  use Modulex\n\n  defbehaviour do\n    @callback say_hello(name :: String.t()) :: String.t()\n  end\n\n  defimplementation do\n    def say_hello(name) do\n      \"Hello, #{name}!\"\n    end\n  end\nend\n```\n\n`Modulex` implements each of the behaviour functions in `MyApplication.Module`. Unless overriden by the application's configuration `[:my_application, :modules, :module]`, the functions will delegate the execution to the implementations in the module `MyApplication.Module.Implementation`, which is automatically generated by `Modulex` with the body of the `defimplementation` block.\n\nWhen used in companion with a mocking library like Mox, you can easily mock the module's implementation in your tests:\n\n```elixir\n// test_helper.exs\n\nMox.defmock(MyApplication.Module.mock_module(), for: MyApplication.Module.behaviour_module())\nMyApplication.Module.put_application_env_module(MyApplication.Module.mock_module())\n```\n\n> **Important:** We strongly recommend the definition of the mocks and the updating of the application environment with them as part of `test_helpers.exs` or any other module that runs before tests start running in parallel. Because the application environment is a global state, updating it from tests might cause unexpected behaviours that are hard to debug.","ref":"readme.html"},{"type":"extras","title":"Installation - modulex","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `modulex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:modulex, \"~> 0.5.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation"}],"content_type":"text/markdown"}